@@ -1,6 +1,6 @@
 /* 
    HTTP Authentication routines
-   Copyright (C) 1999-2003, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2002, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -32,12 +32,10 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
 
 #include <time.h>
 
@@ -81,6 +79,7 @@ typedef enum {
 struct auth_challenge {
     auth_scheme scheme;
     const char *realm;
+    const char *domain;
     const char *nonce;
     const char *opaque;
     unsigned int stale:1; /* if stale=true */
@@ -142,6 +141,9 @@ typedef struct {
     char *nonce;
     char *cnonce;
     char *opaque;
+    /* A list of domain strings */
+    unsigned int domain_count;
+    char **domain;
     auth_qop qop;
     auth_algorithm alg;
     int nonce_count;
@@ -186,6 +188,10 @@ static void clean_session(auth_session *sess)
     NE_FREE(sess->cnonce);
     NE_FREE(sess->opaque);
     NE_FREE(sess->realm);
+    if (sess->domain_count > 0) {
+	split_string_free(sess->domain);
+	sess->domain_count = 0;
+    }
 }
 
 /* Returns cnonce-value. We just use base64(time).
@@ -205,6 +211,56 @@ static int get_credentials(auth_session *sess, char *pwbuf)
 			    sess->username, pwbuf);
 }
 
+static int parse_domain(auth_session *sess, const char *domain) {
+    char **doms;
+    int count, ret;
+
+    doms = split_string_c(domain, ' ', NULL, " \r\n\t", &count);
+    if (doms != NULL) {
+	if (count > 0) {
+	    sess->domain = doms;
+	    sess->domain_count = count;
+	    ret = 0;
+	} else {
+	    free(doms);
+	    ret = -1;
+	}
+    } else {
+	ret = -1;
+    }
+
+    return ret;
+}
+
+/* Returns:
+ *    0: if uri is in NOT in domain of session
+ * else: uri IS in domain of session (or no domain known)
+ */
+static int is_in_domain(auth_session *sess, const char *uri)
+{
+#if 1
+    return 1;
+#else
+    /* TODO: Need proper URI comparison for this to work. */
+    int ret, dom;
+    const char *abs_path;
+    if (sess->domain_count == 0) {
+	NE_DEBUG(NE_DBG_HTTPAUTH, "No domain, presuming okay.\n");
+	return 1;
+    }
+    ret = 0;
+    abs_path = uri_abspath(uri);
+    for (dom = 0; dom < sess->domain_count; dom++) {
+	NE_DEBUG(NE_DBG_HTTPAUTH, "Checking domain: %s\n", sess->domain[dom]);
+	if (uri_childof(sess->domain[dom], abs_path)) {
+	    ret = 1;
+	    break;
+	}
+    }
+    return ret;
+#endif
+}
+
 /* Examine a Basic auth challenge.
  * Returns 0 if an valid challenge, else non-zero. */
 static int 
@@ -286,7 +342,14 @@ static int digest_challenge(auth_session *sess,
     sess->scheme = auth_scheme_digest;
     sess->nonce = ne_strdup(parms->nonce);
     sess->cnonce = get_cnonce();
-    /* TODO: add domain handling. */
+    if (parms->domain) {
+	if (parse_domain(sess, parms->domain)) {
+	    /* TODO: Handle the error? */
+	}
+    } else {
+	sess->domain = NULL;
+	sess->domain_count = 0;
+    }
     if (parms->opaque != NULL) {
 	sess->opaque = ne_strdup(parms->opaque); /* don't strip the quotes */
     }
@@ -582,7 +645,7 @@ static int verify_response(struct auth_request *req, auth_session *sess,
 	    cnonce = val;
 	} else if (strcasecmp(key, "nc") == 0) { 
 	    nc = val;
-	    if (sscanf(val, "%x", &nonce_count) != 1) {
+	    if (sscanf(val, "%d", &nonce_count) != 1) {
 		NE_DEBUG(NE_DBG_HTTPAUTH, "Couldn't find nonce count.\n");
 	    } else {
 		NE_DEBUG(NE_DBG_HTTPAUTH, "Got nonce_count: %d\n", nonce_count);
@@ -736,6 +799,8 @@ static int auth_challenge(auth_session *sess, const char *value)
 	    chall->nonce = val;
 	} else if (strcasecmp(key, "opaque") == 0) {
 	    chall->opaque = val;
+	} else if (strcasecmp(key, "domain") == 0) {
+	    chall->domain = val;
 	} else if (strcasecmp(key, "stale") == 0) {
 	    /* Truth value */
 	    chall->stale = (strcasecmp(val, "true") == 0);
@@ -747,7 +812,7 @@ static int auth_challenge(auth_session *sess, const char *value)
 	    } else {
 		chall->alg = auth_alg_unknown;
 	    }
-	} else if (strcasecmp(key, "qop") == 0) {
+	} else if (strcasecmp(val, "qop") == 0) {
 	    char **qops;
 	    int qop;
 	    qops = split_string(val, ',', NULL, " \r\n\t");
@@ -759,7 +824,6 @@ static int auth_challenge(auth_session *sess, const char *value)
 		    chall->qop_auth_int = 1;
 		}
 	    }
-	    split_string_free(qops);
 	}
     }
 
@@ -859,6 +923,11 @@ static void ah_pre_send(ne_request *r, void *cookie, ne_buffer *request)
 
     if (!sess->can_handle) {
 	NE_DEBUG(NE_DBG_HTTPAUTH, "Not handling session.\n");
+    } else if (!is_in_domain(sess, req->uri)) {
+	/* We have moved out of the authentication domain */
+	NE_DEBUG(NE_DBG_HTTPAUTH, 
+	      "URI %s outside session domain, not handling.\n", req->uri);
+	req->will_handle = 0;
     } else {
 	char *value;
 
@@ -943,6 +1012,17 @@ static void free_auth(void *cookie)
     free(sess);
 }
 
+static void register_hooks(ne_session *sess, auth_session *as, const char *id)
+{
+    ne_hook_create_request(sess, ah_create, as);
+    ne_hook_pre_send(sess, ah_pre_send, as);
+    ne_hook_post_send(sess, ah_post_send, as);
+    ne_hook_destroy_request(sess, ah_destroy, as);
+    ne_hook_destroy_session(sess, free_auth, as);
+
+    ne_set_session_private(sess, id, as);
+}
+
 static void auth_register(ne_session *sess,
 			  const struct auth_class *ahc, const char *id, 
 			  ne_request_auth callback, void *userdata) 
@@ -954,14 +1034,7 @@ static void auth_register(ne_session *sess,
     ahs->sess = sess;
     ahs->spec = ahc;
 
-    /* Register hooks */
-    ne_hook_create_request(sess, ah_create, ahs);
-    ne_hook_pre_send(sess, ah_pre_send, ahs);
-    ne_hook_post_send(sess, ah_post_send, ahs);
-    ne_hook_destroy_request(sess, ah_destroy, ahs);
-    ne_hook_destroy_session(sess, free_auth, ahs);
-
-    ne_set_session_private(sess, id, ahs);
+    register_hooks(sess, ahs, id);
 }
 
 void ne_set_server_auth(ne_session *sess, ne_request_auth callback, 
