@@ -1,6 +1,6 @@
 /* 
    socket handling routines
-   Copyright (C) 1998-2003, Joe Orton <joe@manyfish.co.uk>, 
+   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>, 
    Copyright (C) 1999-2000 Tommi Komulainen <Tommi.Komulainen@iki.fi>
 
    This library is free software; you can redistribute it and/or
@@ -26,15 +26,6 @@
 
 #include "config.h"
 
-#ifdef __hpux
-/* pick up hstrerror */
-#define _XOPEN_SOURCE_EXTENDED 1
-/* don't use the broken getaddrinfo shipped in HP-UX 11.11 */
-#ifdef USE_GETADDRINFO
-#undef USE_GETADDRINFO
-#endif
-#endif
-
 #include <sys/types.h>
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
@@ -112,28 +103,18 @@ typedef struct in_addr ne_inet_addr;
 #define NEON_WRITE(a,b,c) send(a,b,c,0)
 #define NEON_READ(a,b,c) recv(a,b,c,0)
 #define NEON_CLOSE(s) closesocket(s)
-#define ne_errno errno
 #elif defined(WIN32)
 #define NEON_WRITE(a,b,c) send(a,b,c,0)
 #define NEON_READ(a,b,c) recv(a,b,c,0)
 #define NEON_CLOSE(s) closesocket(s)
-#define ne_errno WSAGetLastError()
 #else /* really Unix! */
 #define NEON_WRITE(a,b,c) write(a,b,c)
 #define NEON_READ(a,b,c) read(a,b,c)
 #define NEON_CLOSE(s) close(s)
-#define ne_errno errno
 #endif
 
-#ifdef WIN32
-#define NE_ISRESET(e) ((e) == WSAECONNABORTED || (e) == WSAETIMEDOUT || \
-                       (e) == WSAECONNRESET || (e) == WSAENETRESET)
-#define NE_ISCLOSED(e) ((e) == WSAESHUTDOWN || (e) == WSAENOTCONN)
-#define NE_ISINTR(e) (0)
-#else /* Unix */
-#define NE_ISRESET(e) ((e) == ECONNRESET)
-#define NE_ISCLOSED(e) ((e) == EPIPE)
-#define NE_ISINTR(e) ((e) == EINTR)
+#if !defined(ECONNRESET) && defined(WIN32)
+#define ECONNRESET WSAECONNRESET
 #endif
 
 #ifdef NEON_SSL
@@ -196,21 +177,8 @@ struct ne_sock_addr_s {
 /* set_error: set socket error string to 'str'. */
 #define set_error(s, str) ne_strnzcpy((s)->error, (str), sizeof (s)->error)
 
-/* set_strerror: set socket error to system error string for 'errnum' */
-#ifdef WIN32
-/* Print system error message to given buffer. */
-static void print_error(int errnum, char *buffer, size_t buflen)
-{
-    if (FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM
-                       | FORMAT_MESSAGE_IGNORE_INSERTS,
-                       NULL, (DWORD) errnum, 0, 
-                       buffer, buflen, NULL) == 0)
-        ne_snprintf(buffer, buflen, "Socket error %d", errnum);
-}
-#define set_strerror(s, e) print_error((e), (s)->error, sizeof (s)->error)
-#else /* not WIN32 */
+/* set_strerror: set socket error to system error string for 'e' */
 #define set_strerror(s, e) ne_strerror((e), (s)->error, sizeof (s)->error)
-#endif
 
 void ne_register_progress(ne_socket *sock, ne_progress cb, void *userdata)
 {
@@ -224,55 +192,34 @@ static int prng_seeded = 0;
 /* Initialize SSL library; returns non-zero on failure. */
 static int init_ssl(void)
 {
+    const char *file;
+
     SSL_load_error_strings();
     SSL_library_init();
     PKCS12_PBE_add();  /* ### not sure why this is needed. */
 
-    /* Check whether the PRNG has already been seeded. */
     if (RAND_status() == 1)
+	return 0; /* already seeded internally by OpenSSL */
+
+    NE_DEBUG(NE_DBG_SOCKET, "Trying to seed PRNG...");
+
+    file = getenv("EGDSOCKET");
+    if (file && RAND_egd(file) != -1)
 	return 0;
 
-#ifdef EGD_PATH
-    NE_DEBUG(NE_DBG_SOCKET, "Seeding PRNG from " EGD_PATH "...\n");
-    if (RAND_egd(EGD_PATH) != -1)
-	return 0;
-#elif defined(ENABLE_EGD)
-    {
-	static const char *paths[] = { "/var/run/egd-pool", "/dev/egd-pool",
-				       "/etc/egd-pool", "/etc/entropy" };
-	size_t n;
-	for (n = 0; n < sizeof(paths) / sizeof(char *); n++) {
-	    NE_DEBUG(NE_DBG_SOCKET, "Seeding PRNG from %s...\n", paths[n]);
-	    if (RAND_egd(paths[n]) != -1)
-		return 0;
-	}
+    file = getenv("HOME");
+    if (file) {
+	char path[1024];    
+	ne_snprintf(path, sizeof path, "%s/.entropy", file);
+	if (RAND_egd(path) != -1)
+	    return 0;
     }
-#endif /* EGD_PATH */
 
     NE_DEBUG(NE_DBG_SOCKET, "No entropy source found; could not seed PRNG.\n");
     return -1;
 }
 #endif /* NEON_SSL */
 
-#ifdef USE_GETADDRINFO
-static int ipv6_disabled = 0;
-
-/* On Linux kernels, IPv6 is typically built as a loadable module, and
- * socket(AF_INET6, ...) will fail if this module is not loaded, so
- * the slow AAAA lookups can be avoided for this common case. */
-static void init_ipv6(void)
-{
-    int fd = socket(AF_INET6, SOCK_STREAM, 0);
-    
-    if (fd < 0)
-        ipv6_disabled = 1;
-    else
-        close(fd);
-}
-#endif
-
-static int init_result = 0;
-
 int ne_sock_init(void)
 {
 #ifdef WIN32
@@ -280,10 +227,11 @@ int ne_sock_init(void)
     WSADATA wsaData;
     int err;
 #endif
+    static int result = 0; 
 
-    if (init_result > 0) 
+    if (result > 0) 
 	return 0;
-    else if (init_result < 0)
+    else if (result < 0)
 	return -1;
 
 #ifdef WIN32    
@@ -291,7 +239,7 @@ int ne_sock_init(void)
     
     err = WSAStartup(wVersionRequested, &wsaData);
     if (err != 0) {
-	init_result = -1;
+	result = -1;
 	return -1;
     }
 
@@ -305,20 +253,16 @@ int ne_sock_init(void)
     (void) signal(SIGPIPE, SIG_IGN);
 #endif
 
-#ifdef USE_GETADDRINFO
-    init_ipv6();
-#endif
-
 #ifdef NEON_SSL
     if (init_ssl()) {
-	NE_DEBUG(NE_DBG_SOCKET, "SSL initialization failed; lacking PRNG?\n");
-	init_result = -1;
+	NE_DEBUG(NE_DBG_SOCKET, "SSL initialization failed; lacking PRNG?");
+	result = -1;
 	return -1;
     }
     prng_seeded = 1;
 #endif
 
-    init_result = 1;
+    result = 1;
     return 0;
 }
 
@@ -326,8 +270,7 @@ void ne_sock_exit(void)
 {
 #ifdef WIN32
     WSACleanup();
-#endif
-    init_result = 0;
+#endif	
 }
 
 int ne_sock_block(ne_socket *sock, int n)
@@ -415,10 +358,11 @@ static int readable_raw(ne_socket *sock, int secs)
 	    tvp->tv_sec = secs;
 	    tvp->tv_usec = 0;
 	}
+	errno = 0;
 	ret = select(fdno + 1, &rdfds, NULL, NULL, tvp);
-    } while (ret < 0 && NE_ISINTR(ne_errno));
+    } while (ret < 0 && errno == EINTR);
     if (ret < 0) {
-	set_strerror(sock, ne_errno);
+	set_strerror(sock, errno);
 	return NE_SOCK_ERROR;
     }
     return (ret == 0) ? NE_SOCK_TIMEOUT : 0;
@@ -433,22 +377,22 @@ static ssize_t read_raw(ne_socket *sock, char *buffer, size_t len)
 
     do {
 	ret = NEON_READ(sock->fd, buffer, len);
-    } while (ret == -1 && NE_ISINTR(ne_errno));
+    } while (ret == -1 && errno == EINTR);
 
     if (ret == 0) {
 	set_error(sock, _("Connection closed"));
 	ret = NE_SOCK_CLOSED;
     } else if (ret < 0) {
-	int errnum = ne_errno;
-	ret = NE_ISRESET(errnum) ? NE_SOCK_RESET : NE_SOCK_ERROR;
+	int errnum = errno;
+	ret = (errnum == ECONNRESET) ? NE_SOCK_RESET : NE_SOCK_ERROR;
 	set_strerror(sock, errnum);
     }
 
     return ret;
 }
 
-#define MAP_ERR(e) (NE_ISCLOSED(e) ? NE_SOCK_CLOSED : \
-                    (NE_ISRESET(e) ? NE_SOCK_RESET : NE_SOCK_ERROR))
+#define MAP_ERR(e) ((e) == EPIPE ? NE_SOCK_CLOSED : \
+((e) == ECONNRESET ? NE_SOCK_RESET : NE_SOCK_ERROR))
 
 static ssize_t write_raw(ne_socket *sock, const char *data, size_t length) 
 {
@@ -456,14 +400,12 @@ static ssize_t write_raw(ne_socket *sock, const char *data, size_t length)
     
     do {
 	wrote = NEON_WRITE(sock->fd, data, length);
-        if (wrote > 0) {
-            data += wrote;
-            length -= wrote;
-        }
-    } while ((wrote > 0 || NE_ISINTR(ne_errno)) && length > 0);
+	data += wrote;
+	length -= wrote;
+    } while ((wrote > 0 || errno == EINTR) && length > 0);
 
     if (wrote < 0) {
-	int errnum = ne_errno;
+	int errnum = errno;
 	set_strerror(sock, errnum);
 	return MAP_ERR(errnum);
     }
@@ -507,7 +449,7 @@ static int error_ossl(ne_socket *sock, int sret)
 		ret = NE_SOCK_TRUNC;
 	    } else {
 		/* Other socket error. */
-		err = ne_errno;
+		err = errno;
 		set_strerror(sock, err);
 		ret = MAP_ERR(err);
 	    }
@@ -636,6 +578,7 @@ ne_sock_addr *ne_addr_resolve(const char *hostname, int flags)
 #ifdef USE_GETADDRINFO
     struct addrinfo hints = {0};
     char *pnt;
+    hints.ai_family = AF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
     if (hostname[0] == '[' && ((pnt = strchr(hostname, ']')) != NULL)) {
 	char *hn = ne_strdup(hostname + 1);
@@ -643,11 +586,9 @@ ne_sock_addr *ne_addr_resolve(const char *hostname, int flags)
 #ifdef AI_NUMERICHOST /* this was missing from the RFC2133 API */
 	hints.ai_flags = AI_NUMERICHOST;
 #endif
-        hints.ai_family = AF_INET6;
 	addr->errnum = getaddrinfo(hn, NULL, &hints, &addr->result);
 	free(hn);
     } else {
-        hints.ai_family = ipv6_disabled ? AF_INET : AF_UNSPEC;
 	addr->errnum = getaddrinfo(hostname, NULL, &hints, &addr->result);
     }
 #else /* Use gethostbyname() */
@@ -658,11 +599,7 @@ ne_sock_addr *ne_addr_resolve(const char *hostname, int flags)
     if (laddr == INADDR_NONE) {
 	hp = gethostbyname(hostname);
 	if (hp == NULL) {
-#ifdef WIN32
-	    addr->errnum = WSAGetLastError();
-#else
-            addr->errnum = h_errno;
-#endif
+	    addr->errnum = h_errno;
 	} else if (hp->h_length != sizeof(struct in_addr)) {
 	    /* fail gracefully if somebody set RES_USE_INET6 */
 	    addr->errnum = NO_RECOVERY;
@@ -720,12 +657,9 @@ ne_inet_addr *ne_addr_next(ne_sock_addr *addr)
 
 char *ne_addr_error(const ne_sock_addr *addr, char *buf, size_t bufsiz)
 {
-#ifdef WIN32
-    print_error(addr->errnum, buf, bufsiz);
-#else
     const char *err;
 #ifdef USE_GETADDRINFO
-    /* override horrible generic "Name or service not known" error. */
+    /* over-ride horrible generic "Name or service not known" error. */
     if (addr->errnum == EAI_NONAME)
 	err = _("Host not found");
     else
@@ -736,7 +670,6 @@ char *ne_addr_error(const ne_sock_addr *addr, char *buf, size_t bufsiz)
     err = _("Host not found");
 #endif
     ne_strnzcpy(buf, err, bufsiz);
-#endif /* WIN32 */
     return buf;
 }
 
@@ -780,17 +713,14 @@ static int raw_connect(int fd, ne_inet_addr *addr, unsigned int port)
 {
 #ifdef USE_GETADDRINFO
 #ifdef AF_INET6
-    /* fill in the _family field for AIX 4.3, which forgets to do so. */
     if (addr->ai_family == AF_INET6) {
 	struct sockaddr_in6 *in6 = (struct sockaddr_in6 *)addr->ai_addr;
 	in6->sin6_port = port;
-        in6->sin6_family = AF_INET6;
     } else
 #endif
     if (addr->ai_family == AF_INET) {
 	struct sockaddr_in *in = (struct sockaddr_in *)addr->ai_addr;
 	in->sin_port = port;
-        in->sin_family = AF_INET;
     } /* else something weird, and the connect should fail... */
     return connect(fd, addr->ai_addr, addr->ai_addrlen);
 #else
