@@ -156,7 +156,7 @@ static int expect_response(const char *expect, server_fn fn, void *userdata)
     return OK;
 }
 
-#define EMPTY_RESP RESP200 "Content-Length: 0\r\n\r\n"
+#define FOLLOW_REQ RESP200 "Content-Length: 0\r\n\r\n"
 
 /* Process a request with given method and response, expecting to get
  * a zero-length response body.  A second request is sent down the
@@ -166,12 +166,12 @@ static int expect_response(const char *expect, server_fn fn, void *userdata)
 static int expect_no_body(const char *method, const char *resp)
 {
     ne_session *sess = ne_session_create("http", "localhost", 7777);
-    ne_request *req = ne_request_create(sess, method, "/first");
+    ne_request *req = ne_request_create(sess, method, "/foobar");
     ssize_t ret;
-    char *r = ne_malloc(strlen(resp) + sizeof(EMPTY_RESP));
+    char *r = ne_malloc(strlen(resp) + sizeof(FOLLOW_REQ));
     
     strcpy(r, resp);
-    strcat(r, EMPTY_RESP);
+    strcat(r, FOLLOW_REQ);
     ON(spawn_server(7777, single_serve_string, r));
     free(r);
 
@@ -182,7 +182,7 @@ static int expect_no_body(const char *method, const char *resp)
 
     /* process following request; makes sure that nothing extra has
      * been eaten by the first request. */
-    ONV(any_request(sess, "/second"),
+    ONV(any_request(sess, "/foo"),
 	("second request on connection failed: %s",ne_get_error(sess)));
 
     ON(await_server());
@@ -257,18 +257,6 @@ static int no_body_HEAD(void)
 			  "Content-Length: 5\r\n\r\n");
 }
 
-static int no_body_empty_clength(void)
-{
-    return expect_no_body("GET", "HTTP/1.1 200 OK\r\n"
-			  "Content-Length:\r\n\r\n");
-}
-
-static int no_body_bad_clength(void)
-{
-    return expect_no_body("GET", "HTTP/1.1 200 OK\r\n"
-			  "Content-Length: foobar\r\n\r\n");
-}
-
 static int no_headers(void)
 {
     return expect_response("abcde", single_serve_string,
@@ -1228,19 +1216,6 @@ static int fail_lookup(void)
     return fail_noserver("no.such.domain", 7777, NE_LOOKUP);
 }
 
-/* neon 0.23.0 to 0.23.3: if a nameserver lookup failed, subsequent
- * requests on the session would crash. */
-static int fail_double_lookup(void)
-{
-     ne_session *sess = ne_session_create("http", "nohost.example.com", 80);
-     ONN("request did not give lookup failure",
-	 any_request(sess, "/foo") != NE_LOOKUP);
-     ONN("second request did not give lookup failure",
-	 any_request(sess, "/bar") != NE_LOOKUP);
-     ne_session_destroy(sess);
-     return OK;
-}
-
 static int fail_connect(void)
 {
     return fail_noserver("localhost", 7777, NE_CONNECT);
@@ -1360,104 +1335,6 @@ static int fail_statusline(void)
     return OK;    
 }
 
-#define LEN (9000)
-static int fail_long_header(void)
-{
-    char resp[LEN + 500] = "HTTP/1.1 200 OK\r\n"
-	"Server: fish\r\n";
-    size_t len = strlen(resp);
-    
-    /* add a long header */
-    memset(resp + len, 'a', LEN);
-    resp[len + LEN] = '\0';
-    
-    strcat(resp, "\r\n\r\n");
-
-    return fail_request(0, single_serve_string, resp, 0);
-}
-
-#if 0
-static int versions(void)
-{
-    ne_session *sess;
-
-    CALL(make_session(&sess, single_serve_string, 
-		      "HTTP/1.1 200 OK\r\n"
-		      "Content-Length: 0\r\n\r\n"
-
-		      "HTTP/1.0 200 OK\r\n"
-		      "Content-Length: 0\r\n\r\n"));
-    
-    CALL(any_request(sess, "/http11"));
-	 
-    ONN("did not detect HTTP/1.1 compliance",
-	ne_version_pre_http11(sess) == 0);
-	 
-    CALL(any_request(sess, "/http10"));
-
-    ONN("did not detect lack of HTTP/1.1 compliance",
-	ne_version_pre_http11(sess) == 0);
-
-    ne_session_destroy(sess);
-
-    return OK;
-}
-#endif
-
-struct cr_args {
-    const char *method, *uri;
-    int result;
-};
-
-static void hk_createreq(ne_request *req, void *userdata,
-			 const char *method, const char *requri)
-{
-    struct cr_args *args = userdata;
-    
-    args->result = 1; /* presume failure */
-    
-    if (strcmp(args->method, method))
-	t_context("Hook got method %s not %s", method, args->method);
-    else if (strcmp(args->uri, requri))
-	t_context("Hook got Req-URI %s not %s", requri, args->uri);
-    else
-	args->result = 0;
-}
-
-static int hook_create_req(void)
-{
-    ne_session *sess;
-    struct cr_args args;
-
-    CALL(make_session(&sess, single_serve_string, EMPTY_RESP EMPTY_RESP));
-
-    ne_hook_create_request(sess, hk_createreq, &args);
-
-    args.method = "GET";
-    args.uri = "/foo";
-    args.result = -1;
-
-    CALL(any_request(sess, "/foo"));
-    
-    ONN("first hook never called", args.result == -1);
-    if (args.result) return FAIL;
-
-    args.uri = "http://localhost:7777/bar";
-    args.result = -1;
-    
-    /* force use of absoluteURI in request-uri */
-    ne_session_proxy(sess, "localhost", 7777);
-
-    CALL(any_request(sess, "/bar"));
-    
-    ONN("second hook never called", args.result == -1);
-    if (args.result) return FAIL;
-
-    ne_session_destroy(sess);
-
-    return OK;    
-}
-
 ne_test tests[] = {
     T(lookup_localhost),
     T(single_get_clength),
@@ -1466,8 +1343,6 @@ ne_test tests[] = {
     T(no_body_204),
     T(no_body_304),
     T(no_body_HEAD),
-    T(no_body_empty_clength),
-    T(no_body_bad_clength),
     T(no_headers),
     T(chunks),
     T(te_header),
@@ -1511,19 +1386,13 @@ ne_test tests[] = {
     T(not_http),
     T(fail_eof_continued),
     T(fail_eof_headers),
-    T(fail_long_header),
     T(read_timeout),
     T(fail_lookup),
-    T(fail_double_lookup),
     T(fail_connect),
     T(proxy_no_resolve),
     T(fail_chunksize),
     T(abort_respbody),
     T(retry_after_abort),
     T(fail_statusline),
-#if 0
-    T(versions),
-#endif
-    T(hook_create_req),
     T(NULL)
 };
