@@ -125,6 +125,19 @@ static int serve_ssl(ne_socket *sock, void *ud)
     return OK;
 }
 
+static int serve_response(ne_socket *sock, void *ud)
+{
+    const char *resp = ud;
+
+    ONN("failed to load certificate",
+	!SSL_CTX_use_certificate_file(server_ctx, 
+				      SERVER_CERT, SSL_FILETYPE_PEM));
+
+    CALL(do_ssl_response(sock, server_ctx, NULL, resp, 0));
+
+    return OK;
+}    
+
 static int serve_response_unclean(ne_socket *sock, void *ud)
 {
     const char *resp = ud;
@@ -428,13 +441,6 @@ static int multi_commonName(void)
     return accept_signed_cert("twocn.cert");
 }
 
-/* regression test for neon <= 0.23.4 where if commonName was the first
- * RDN in the subject DN, it was ignored. */
-static int commonName_first(void)
-{
-    return accept_signed_cert("cnfirst.cert");
-}
-
 /* Verify callback which checks that the certificate has the predetermined
  * values (as per makekeys.sh). */
 static int check_DNs(void *userdata, int fs, const ne_ssl_certificate *cert)
@@ -828,7 +834,6 @@ ne_test tests[] = {
     T(notdns_altname),
 
     T(multi_commonName),
-    T(commonName_first),
 
     T(fail_wrongCN),
     T(fail_expired),
