@@ -76,7 +76,7 @@ static int parse_match(struct ne_xml_elm *elms,
 
     ne_xml_parse(p, doc, strlen(doc));
     
-    ONV(!ne_xml_valid(p), ("parse failed: %s", ne_xml_get_error(p)));
+    ONN("parse failed", !ne_xml_valid(p));
     
     ONV(strcmp(result, buf->data),
 	("result mismatch: %s not %s", buf->data, result));
@@ -119,12 +119,6 @@ static int matches(void)
 	  "<{foo}widget xmlns='foo'><{}smidgen xmlns=''>" 
 	  E("", "norman") 
 	  "</{}smidgen></{foo}widget>" },
-	/* regression test for neon <= 0.23.5 with libxml2, where the
-	 * "dereference entities" flag was not set by default. */
-	{ PFX "<widget foo=\"no&amp;body\"/>",
-	  "<{}widget foo='no&body'></{}widget>" },
-	{ PFX "<widget foo=\"no&#x20;body\"/>",
-	  "<{}widget foo='no body'></{}widget>" },
 	{ NULL, NULL }
     };
     struct ne_xml_elm elms[] = {
